@@ -35,8 +35,11 @@ from Orange.widgets.utils.annotated_data import (
     create_annotated_table, ANNOTATED_DATA_SIGNAL_NAME
 )
 from Orange.widgets.utils.itemmodels import VariableListModel
-from .owscatterplotgraph import LegendItem, legend_anchor_pos
+from Orange.widgets.utils.plot import OWPlotGUI
+from Orange.widgets.visualize.owscatterplotgraph import LegendItem, \
+    legend_anchor_pos
 from Orange.widgets.utils import classdensity
+from Orange.widgets.widget import Input, Output
 from Orange.canvas import report
 
 
@@ -182,23 +185,29 @@ class OWLinearProjection(widget.OWWidget):
     icon = "icons/LinearProjection.svg"
     priority = 240
 
-    inputs = [("Data", Table, "set_data", widget.Default),
-              ("Data Subset", Table, "set_subset_data")]
+    class Inputs:
+        data = Input("Data", Table, default=True)
+        data_subset = Input("Data Subset", Table)
+
 #              #TODO: Allow for axes to be supplied from an external source.
 #               ("Projection", numpy.ndarray, "set_axes"),]
-    outputs = [("Selected Data", Table, widget.Default),
-               (ANNOTATED_DATA_SIGNAL_NAME, Table)]
 
+    class Outputs:
+        selected_data = Output("Selected Data", Table, default=True)
+        annotated_data = Output(ANNOTATED_DATA_SIGNAL_NAME, Table)
+
+    settings_version = 2
     settingsHandler = settings.DomainContextHandler()
 
     variable_state = settings.ContextSetting({})
 
-    color_index = settings.ContextSetting(0)
-    shape_index = settings.ContextSetting(0)
-    size_index = settings.ContextSetting(0)
+    attr_color = settings.ContextSetting(None, exclude_metas=False)
+    attr_label = settings.ContextSetting(None, exclude_metas=False)
+    attr_shape = settings.ContextSetting(None, exclude_metas=False)
+    attr_size = settings.ContextSetting(None, exclude_metas=False)
 
-    point_size = settings.Setting(10)
-    alpha_value = settings.Setting(255)
+    point_width = settings.Setting(10)
+    alpha_value = settings.Setting(128)
     jitter_value = settings.Setting(0)
 
     class_density = settings.Setting(False)
@@ -285,79 +294,6 @@ class OWLinearProjection(widget.OWWidget):
 
         box1.layout().addWidget(view)
 
-        box = gui.vBox(self.controlArea, box=True)
-        box.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Maximum)
-
-        self.colorvar_model = VariableListModel(parent=self, enable_dnd=True)
-        self.shapevar_model = VariableListModel(parent=self, enable_dnd=True)
-        self.sizevar_model = VariableListModel(parent=self, enable_dnd=True)
-
-        form = QFormLayout(
-            formAlignment=Qt.AlignLeft,
-            labelAlignment=Qt.AlignLeft,
-            fieldGrowthPolicy=QFormLayout.AllNonFixedFieldsGrow,
-            spacing=8
-        )
-        box.layout().addLayout(form)
-
-        cb = gui.comboBox(box, self, "color_index",
-                          callback=self._on_color_change,
-                          contentsLength=10)
-        cb.setModel(self.colorvar_model)
-        form.addRow("Color:", cb)
-
-        box = gui.vBox(None)
-        hbox = gui.indentedBox(box, orientation=Qt.Horizontal)
-        gui.widgetLabel(hbox, "Opacity: ")
-        alpha_slider = QSlider(
-            Qt.Horizontal, minimum=10, maximum=255, pageStep=25,
-            tickPosition=QSlider.TicksBelow, value=self.alpha_value)
-        alpha_slider.valueChanged.connect(self._set_alpha)
-        hbox.layout().addWidget(alpha_slider)
-
-        self.cb_class_density = gui.checkBox(
-            gui.indentedBox(box), self, "class_density",
-            label="Show class density", callback=self._update_density)
-        gui.separator(box)
-        form.addRow(box)
-
-        cb = gui.comboBox(box, self, "shape_index",
-                          callback=self._on_shape_change,
-                          contentsLength=10)
-
-        cb.setModel(self.shapevar_model)
-        form.addRow("Shape:", cb)
-
-        cb = gui.comboBox(box, self, "size_index",
-                          callback=self._on_size_change,
-                          contentsLength=10)
-
-        cb.setModel(self.sizevar_model)
-        form.addRow("Size:", cb)
-
-        size_slider = QSlider(
-            Qt.Horizontal, minimum=3, maximum=30, value=self.point_size,
-            pageStep=3,
-            tickPosition=QSlider.TicksBelow)
-        size_slider.valueChanged.connect(self._set_size)
-        form.addRow("", size_slider)
-
-        cb = self.jitter_combo = gui.comboBox(
-            box, self, "jitter_value",
-            items=["None", "0.01 %", "0.1 %", "0.5 %", "1 %", "2 %"],
-            callback=self._invalidate_plot)
-        form.addRow("Jittering:", cb)
-
-        toolbox = gui.vBox(self.controlArea, "Zoom/Select")
-        toollayout = QHBoxLayout()
-        toolbox.layout().addLayout(toollayout)
-
-        gui.auto_commit(self.controlArea, self, "auto_commit", "Send Selection",
-                        auto_label="Send Automatically")
-
-        self.controlArea.setSizePolicy(
-            QSizePolicy.Preferred, QSizePolicy.Expanding)
-
         # Main area plot
         self.view = pg.GraphicsView(background="w")
         self.view.setRenderHint(QPainter.Antialiasing, True)
@@ -366,8 +302,41 @@ class OWLinearProjection(widget.OWWidget):
         self.viewbox.setAspectLocked(True)
         self.viewbox.grabGesture(Qt.PinchGesture)
         self.view.setCentralItem(self.viewbox)
-
         self.mainArea.layout().addWidget(self.view)
+        self.replot = None
+
+        g = OWPlotGUI(self)
+        g.point_properties_box(self.controlArea)
+        self.controls.attr_label.parent().setVisible(False)
+        self.models = g.points_models
+
+        box = gui.widgetBox(self.controlArea, "Plot")
+        form = QFormLayout(
+            formAlignment=Qt.AlignLeft,
+            labelAlignment=Qt.AlignLeft,
+            fieldGrowthPolicy=QFormLayout.AllNonFixedFieldsGrow,
+            spacing=8
+        )
+        box.layout().addLayout(form)
+
+        self.jitter_combo = gui.comboBox(
+            box, self, "jitter_value",
+            items=["None", "0.01 %", "0.1 %", "0.5 %", "1 %", "2 %"],
+            callback=self._invalidate_plot)
+        form.addRow("Jittering:", self.jitter_combo)
+
+        self.cb_class_density = gui.checkBox(
+            box, self, "class_density", "", callback=self._update_density)
+        form.addRow("Class density", self.cb_class_density)
+
+        toolbox = gui.vBox(self.controlArea, "Zoom/Select")
+        toollayout = QHBoxLayout()
+        toolbox.layout().addLayout(toollayout)
+
+        self.controlArea.layout().addStretch(1)
+
+        gui.auto_commit(self.controlArea, self, "auto_commit", "Send Selection",
+                        auto_label="Send Automatically")
 
         self.selection = PlotSelectionTool(self)
         self.selection.setViewBox(self.viewbox)
@@ -456,18 +425,8 @@ class OWLinearProjection(widget.OWWidget):
         self.data = None
         self._subset_mask = None
         self._selection_mask = None
-
         self.varmodel_selected[:] = []
         self.varmodel_other[:] = []
-
-        self.colorvar_model[:] = []
-        self.sizevar_model[:] = []
-        self.shapevar_model[:] = []
-
-        self.color_index = 0
-        self.size_index = 0
-        self.shape_index = 0
-
         self.clear_plot()
 
     def clear_item(self):
@@ -507,6 +466,16 @@ class OWLinearProjection(widget.OWWidget):
             QApplication.postEvent(self, QEvent(self.ReplotRequest),
                                    Qt.LowEventPriority - 10)
 
+    def init_attr_values(self):
+        domain = self.data and len(self.data) and self.data.domain or None
+        for model in self.models:
+            model.set_domain(domain)
+        self.attr_color = domain and self.data.domain.class_var or None
+        self.attr_shape = None
+        self.attr_size = None
+        self.attr_label = None
+
+    @Inputs.data
     def set_data(self, data):
         """
         Set the input dataset.
@@ -526,6 +495,7 @@ class OWLinearProjection(widget.OWWidget):
                 data_sample.download_data(2000, partial=True)
                 data = Table(data_sample)
         self.data = data
+        self.init_attr_values()
         if data is not None and len(data):
             self._initialize(data)
             # get the default encoded state, replacing the position with Inf
@@ -554,19 +524,9 @@ class OWLinearProjection(widget.OWWidget):
             )
             self.varmodel_selected[:] = selected
             self.varmodel_other[:] = other
-
-            def clip_index(value, maxv):
-                return max(0, min(value, maxv))
-
-            self.color_index = clip_index(
-                self.color_index, len(self.colorvar_model) - 1)
-            self.shape_index = clip_index(
-                self.shape_index, len(self.shapevar_model) - 1)
-            self.size_index = clip_index(
-                self.size_index, len(self.sizevar_model) - 1)
-
             self._invalidate_plot()
 
+    @Inputs.data_subset
     def set_subset_data(self, subset):
         """
         Set the supplementary input subset dataset.
@@ -620,57 +580,15 @@ class OWLinearProjection(widget.OWWidget):
         return [[var for _, var in sorted(newlist, key=itemgetter(0))]
                 for newlist in newlists]
 
-    def color_var(self):
-        """
-        Current selected color variable or None (if not selected).
-        """
-        if 1 <= self.color_index < len(self.colorvar_model):
-            return self.colorvar_model[self.color_index]
-        else:
-            return None
-
-    def size_var(self):
-        """
-        Current selected size variable or None (if not selected).
-        """
-        if 1 <= self.size_index < len(self.sizevar_model):
-            return self.sizevar_model[self.size_index]
-        else:
-            return None
-
-    def shape_var(self):
-        """
-        Current selected shape variable or None (if not selected).
-        """
-        if 1 <= self.shape_index < len(self.shapevar_model):
-            return self.shapevar_model[self.shape_index]
-        else:
-            return None
-
     def _initialize(self, data):
         # Initialize the GUI controls from data's domain.
-        all_vars = list(data.domain.variables)
         cont_vars = [var for var in data.domain.variables
                      if var.is_continuous]
-        disc_vars = [var for var in data.domain.variables
-                     if var.is_discrete]
-        shape_vars = [var for var in disc_vars
-                      if len(var.values) <= len(ScatterPlotItem.Symbols) - 1]
-
         self.warning("Plotting requires continuous features.",
                      shown=not len(cont_vars))
-
-        self.all_vars = data.domain.variables
         self.varmodel_selected[:] = cont_vars[:3]
         self.varmodel_other[:] = cont_vars[3:]
 
-        self.colorvar_model[:] = ["Same color"] + all_vars
-        self.sizevar_model[:] = ["Same size"] + cont_vars
-        self.shapevar_model[:] = ["Same shape"] + shape_vars
-
-        if data.domain.has_discrete_class:
-            self.color_index = all_vars.index(data.domain.class_var) + 1
-
     def __activate_selection(self):
         view = self.other_view
         model = self.varmodel_other
@@ -764,9 +682,8 @@ class OWLinearProjection(widget.OWWidget):
             self.viewbox.setRange(QRectF(-1.05, -1.05, 2.1, 2.1))
         self._update_legend()
 
-        color_var = self.color_var()
         if self.class_density and \
-                color_var is not None and color_var.is_discrete:
+                self.attr_color is not None and self.attr_color.is_discrete:
             [min_x, max_x], [min_y, max_y] = self.viewbox.viewRange()
             rgb_data = [brush.color().getRgb()[:3] for brush in brush_data]
             self._density_img = classdensity.class_density_image(
@@ -774,16 +691,15 @@ class OWLinearProjection(widget.OWWidget):
             self.viewbox.addItem(self._density_img)
 
     def _color_data(self, mask=None):
-        color_var = self.color_var()
-        if color_var is not None:
-            color_data, _ = self._get_data(color_var, dtype=float)
-            if color_var.is_continuous:
+        if self.attr_color is not None:
+            color_data, _ = self._get_data(self.attr_color, dtype=float)
+            if self.attr_color.is_continuous:
                 color_data = plotutils.continuous_colors(
-                    color_data, None, *color_var.colors)
+                    color_data, None, *self.attr_color.colors)
             else:
                 color_data = plotutils.discrete_colors(
-                    color_data, len(color_var.values),
-                    color_index=color_var.colors
+                    color_data, len(self.attr_color.values),
+                    color_index=self.attr_color.colors
                 )
             if mask is not None:
                 color_data = color_data[mask]
@@ -852,8 +768,7 @@ class OWLinearProjection(widget.OWWidget):
         else:
             self._item.setBrush(brush[self._item._mask])
 
-        color_var = self.color_var()
-        if color_var is not None and color_var.is_discrete:
+        if self.attr_color is not None and self.attr_color.is_discrete:
             self.cb_class_density.setEnabled(True)
             if self.class_density:
                 self._setup_plot(reset_view=False)
@@ -864,14 +779,14 @@ class OWLinearProjection(widget.OWWidget):
         self._update_legend()
 
     def _shape_data(self, mask):
-        shape_var = self.shape_var()
-        if shape_var is None:
+        if self.attr_shape is None:
             shape_data = numpy.array(["o"] * len(self.data))
         else:
-            assert shape_var.is_discrete
+            assert self.attr_shape.is_discrete
             symbols = numpy.array(list(ScatterPlotItem.Symbols))
             max_symbol = symbols.size - 1
-            shapeidx, shape_mask = column_data(self.data, shape_var, dtype=int)
+            shapeidx, shape_mask = column_data(self.data, self.attr_shape,
+                                               dtype=int)
             shapeidx[shape_mask] = max_symbol
             shapeidx[~shape_mask] %= max_symbol -1
             shape_data = symbols[shapeidx]
@@ -888,24 +803,23 @@ class OWLinearProjection(widget.OWWidget):
         self._update_legend()
 
     def _size_data(self, mask=None):
-        size_var = self.size_var()
-        if size_var is None:
-            size_data = numpy.full((len(self.data),), self.point_size,
+        if self.attr_size is None:
+            size_data = numpy.full((len(self.data),), self.point_width,
                                    dtype=float)
         else:
             nan_size = OWLinearProjection.MinPointSize - 2
-            size_data, size_mask = self._get_data(size_var, dtype=float)
+            size_data, size_mask = self._get_data(self.attr_size, dtype=float)
             size_data_valid = size_data[~size_mask]
             if size_data_valid.size:
                 smin, smax = numpy.min(size_data_valid), numpy.max(size_data_valid)
                 sspan = smax - smin
             else:
-                sspan = smax = smin = 0
+                sspan = smin = 0
             size_data[~size_mask] -= smin
             if sspan > 0:
                 size_data[~size_mask] /= sspan
             size_data = \
-                size_data * self.point_size + OWLinearProjection.MinPointSize
+                size_data * self.point_width + OWLinearProjection.MinPointSize
             size_data[size_mask] = nan_size
         if mask is None:
             return size_data
@@ -917,6 +831,10 @@ class OWLinearProjection(widget.OWWidget):
             return
         self.set_size(self._size_data(mask=None))
 
+    update_point_size = update_sizes = _on_size_change
+    update_alpha_value = update_colors = _on_color_change
+    update_shapes = _on_shape_change
+
     def _update_density(self):
         self._setup_plot(reset_view=False)
 
@@ -931,7 +849,7 @@ class OWLinearProjection(widget.OWWidget):
 
         legend.clear()
 
-        color_var, shape_var = self.color_var(), self.shape_var()
+        color_var, shape_var = self.attr_color, self.attr_shape
         if color_var is not None and not color_var.is_discrete:
             color_var = None
         assert shape_var is None or shape_var.is_discrete
@@ -1029,19 +947,15 @@ class OWLinearProjection(widget.OWWidget):
             if len(indices) > 0:
                 subset = self.data[indices]
 
-        self.send("Selected Data", subset)
-        self.send(ANNOTATED_DATA_SIGNAL_NAME,
-                  create_annotated_table(self.data, indices))
+        self.Outputs.selected_data.send(subset)
+        self.Outputs.annotated_data.send(create_annotated_table(self.data, indices))
 
     def send_report(self):
         self.report_plot(name="", plot=self.viewbox.getViewBox())
         caption = report.render_items_vert((
-            ("Colors",
-             self.color_index > 0 and self.colorvar_model[self.color_index]),
-            ("Shape",
-             self.shape_index > 0 and self.shapevar_model[self.shape_index]),
-            ("Size",
-             self.size_index > 0 and self.sizevar_model[self.size_index])
+            ("Colors", self.attr_color),
+            ("Shape", self.attr_shape),
+            ("Size", self.attr_size)
         ))
         jitter_caption = report.render_items_vert(
             (("Jittering",
@@ -1049,6 +963,24 @@ class OWLinearProjection(widget.OWWidget):
         caption = ";<br/>".join(x for x in (caption, jitter_caption) if x)
         self.report_caption(caption)
 
+    @classmethod
+    def migrate_settings(cls, settings_, version):
+        if version < 2:
+            settings_["point_width"] = settings_["point_size"]
+
+    @classmethod
+    def migrate_context(cls, context, version):
+        if version < 2:
+            domain = context.ordered_domain
+            c_domain = [t for t in context.ordered_domain if t[1] == 2]
+            d_domain = [t for t in context.ordered_domain if t[1] == 1]
+            for d, old_val, new_val in ((domain, "color_index", "attr_color"),
+                                        (d_domain, "shape_index", "attr_shape"),
+                                        (c_domain, "size_index", "attr_size")):
+                index = context.values[old_val][0] - 1
+                context.values[new_val] = (d[index][0], d[index][1] + 100) \
+                    if 0 <= index < len(d) else None
+
 
 class PlotTool(QObject):
     """
